"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const taskId_1 = require("./taskId");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const logger_1 = __importDefault(require("../logger"));
const taskWrapper_1 = require("./taskWrapper");
const abortSignal_1 = require("./abortSignal");
function npmTask(taskId, context) {
    const [pkg, task] = taskId_1.getPackageTaskFromId(taskId);
    const { allPackages, queue } = context;
    const prefix = `${pkg} ${task}`;
    return queue.add(() => taskWrapper_1.taskWrapper(taskId, () => new Promise((resolve, reject) => {
        if (!allPackages[pkg].scripts[task]) {
            logger_1.default.info(taskId, `Empty script detected, skipping`);
            return resolve();
        }
        const npmArgs = [
            ...context.nodeArgs,
            path_1.default.join(path_1.default.dirname(process.execPath), "node_modules/npm/bin/npm-cli.js"),
            "run",
            task,
            "--",
            ...context.args,
        ];
        logger_1.default.verbose(taskId, `Running ${[process.execPath, ...npmArgs].join(" ")}`);
        const cp = child_process_1.spawn(process.execPath, npmArgs, {
            cwd: path_1.default.dirname(allPackages[pkg].packageJsonPath),
            stdio: "pipe",
        });
        context.events.once("abort", terminate);
        cp.stdout.on("data", (data) => {
            if (!cp.killed) {
                data
                    .toString()
                    .split(/\n/)
                    .forEach((line) => {
                    logger_1.default.verbose(taskId, line.trim());
                });
            }
        });
        cp.stderr.on("data", (data) => {
            if (!cp.killed) {
                data
                    .toString()
                    .split(/\n/)
                    .forEach((line) => {
                    logger_1.default.verbose(taskId, line.trim());
                });
            }
        });
        cp.on("exit", (code) => {
            context.events.off("off", terminate);
            if (code === 0) {
                return resolve();
            }
            context.measures.failedTask = taskId;
            abortSignal_1.abort(context);
            reject();
        });
        function terminate() {
            queue.pause();
            queue.clear();
            cp.kill("SIGKILL");
        }
    }), context));
}
exports.npmTask = npmTask;
